/* **********************************************************
 * Copyright (C) 1999-2000, VMware, Inc.
 * All Rights Reserved.
 * $Id: vmcontrol.h,v 1.13.6.1 2000/12/01 11:52:48 brettv Exp $*
 * **********************************************************/

#ifndef _VMCONTROL_H_
#define _VMCONTROL_H_

/*
 * vmcontrol.h --
 *
 *	External (public) C VMControl header file.
 *
 *      This file will be distributed outside VMware with the binary
 *      libraries that implement the VMware Server Control Interface.
 *      
 *      This file can also be run through h2xs to create stubs for the
 *      Perl API.  These stubs have been edited to customize them and
 *      are check in under ../perl.  (That is, h2xs can be run
 *      manually, but is not run during the build process)
 *      
 *      The syntax for running h2xs to generate stubs for our three
 *      packages is as follows:
        
        h2xs -p VMControl_Server -v $VERSION -n VMware::Control::Server \
          -M '^VMControl_Server' -Axf `pwd`/../public/vmcontrol.h
        
        h2xs -O -p VMControl_VM -v $VERSION -n VMware::Control::VM \
          -M '^VMControl_VM' -Axf `pwd`/../public/vmcontrol.h
        
        h2xs -O -p VMControl_ -v $VERSION -n VMware::Control \
          -M '^VMControl_(?!VM)(?!Server)' -Axf `pwd`/../public/vmcontrol.h

 *      The syntax for running h2xs to generate stubs for the #defined
 *      constants is as follows:

        h2xs -p VMControl_ -v $VERSION -n VMware::Control \
          -M '^VM_E_' -xf `pwd`/../public/vmcontrol.h
        
 *      Then, manually merge new or changed functions into the .pm and
 *      .xs files in apps/perl.  The other generated files (Changes,
 *      MANIFEST, typemap, Makefile.PL, test.pl) will not contain
 *      anything useful.
 */

// This version number should correspond to the version in the .pm files
#define VMCONTROL_VERSION "0.02"

#define VMAUTHD_DEFAULT_PORT 902

#define MAX_CONFIG_FILES 256

// Timeout for responses from vmauthd and VMX, in microseconds:
#define VMCONTROL_DEFAULT_TIMEOUT (30*1000*1000)  // 30 seconds

#define VM_E_SUCCESS            0  // Operation succeeded
#define VM_E_NOMEM             -1  // Could not allocate memory
#define VM_E_NETFAIL           -2  // Network failure, or could not connect
                                   // to remote host
#define VM_E_INVALIDARGS       -3  // Invalid arguments
#define VM_E_NOACCESS          -4  // Permission denied for this operation
#define VM_E_NOTCONNECTED      -5  // The VMserver or VM struct is invalid
                                   // (there is no connection established when
                                   // there should be)
#define VM_E_BADVERSION        -6  // Version mismatch with vmauthd or VMX
#define VM_E_TIMEOUT           -7  // Timeout talking to vmauthd or VMX
#define VM_E_BADSTATE          -8 // The operation is not valid for the current
                                  // state the VM is in (e.g. Stop() on a
                                  // powered off machine)
#define VM_E_NOEXECVMAUTHD     -9  // Could not exec vmauthd (locally)
#define VM_E_NOPROPERTY       -10  // No such property
#define VM_E_NOSUCHVM         -11  // No such VM
#define VM_E_ALREADYCONNECTED -12  // Already connected
#define VM_E_DISCONNECT       -13  // Connection lost
#define VM_E_BADRESPONSE      -14  // Unexpected response from VMAuthd
#define VM_E_GARBAGE          -15  // Garbage received on connection
                                   // VMAuthd protocol error
#define VM_E_NEEDINPUT        -16  // The VM needs the user to answer
                                   // a question

#define VM_E_UNSPECIFIED  -999  // Unknown error XXX should get rid of these

// Default location if authd.fullpath isn't set
#define VMAUTHD_LOCATION "/usr/sbin/vmware-authd"

typedef struct VMControlServer VMControlServer;
typedef struct VMControlVM VMControlVM;

//XXX #include "vm_basic_types.h" //for Bool, TRUE, FALSE
#ifndef _VM_BASIC_TYPES_H_
typedef char           Bool;

#ifndef FALSE
#define FALSE          0
#endif

#ifndef TRUE
#define TRUE           1
#endif
#endif

/* 
 * VMControl -- top level functions
 * --------------------------------
 */

Bool VMControl_Init(void);
const char *VMControl_Version(void);
const char *VMControl_GetErrorMessage(int err);


/*
 * VMControl Server -- functions that act on a VMControlServer object
 * -----------------------------------------------------------
 */

char VMControl_ServerInit(void);

VMControlServer *
VMControl_ServerNew(const char *hostname, int port, const char *username,
		  const char *password);
int VMControl_ServerGetLastError(VMControlServer *s, char **cptr);
Bool VMControl_ServerConnect(VMControlServer *s);
void VMControl_ServerDisconnect(VMControlServer *s);
void VMControl_ServerDestroy(VMControlServer *s);
Bool VMControl_ServerIsConnected(VMControlServer *s);
char **VMControl_ServerEnumerate(VMControlServer *s);
Bool VMControl_ServerKill(VMControlServer *s, const char *vm); // Not implemented
char *VMControl_ServerExec(VMControlServer *s, const char *xmlargs);

/*
 * Not sure whether these will be in the final API:
 */
const char *VMControl_ServerGetAsString(VMControlServer *s, const char *property);  // Not implemented
Bool VMControl_ServerRegister(VMControlServer *s, const char *configpath);

/* 
 * VMControl VM -- functions that act on a VMControlVM object
 * -------------------------------------------------
 */

char VMControl_VMInit(void);

VMControlVM *VMControl_VMNew(VMControlServer *s, const char *config);
int VMControl_VMGetLastError(VMControlVM *vm, char **cptr);
Bool VMControl_VMConnect(VMControlVM *vm);
void VMControl_VMDisconnect(VMControlVM *vm);
void VMControl_VMDestroy(VMControlVM *vm);

Bool VMControl_VMIsConnected(VMControlVM *vm);
Bool VMControl_VMAnswerQuestion(VMControlVM *vm, int sequence, int choicenum);
Bool VMControl_VMStart(VMControlVM *vm);
Bool VMControl_VMRequestStop(VMControlVM *vm, Bool force);
Bool VMControl_VMRequestReset(VMControlVM *vm, Bool force);
Bool VMControl_VMStop(VMControlVM *vm, Bool force);
Bool VMControl_VMReset(VMControlVM *vm, Bool force);
Bool VMControl_VMSuspendToDisk(VMControlVM *vm);
char ** VMControl_VMInfo(VMControlVM *vm);
Bool VMControl_VMSaveAs(VMControlVM *vm, const char *config);  // Not implemented
Bool VMControl_VMExit(VMControlVM *vm);  // Not implemented

/*
 * Not sure whether these will be in the final API:
 */
char * VMControl_VMGetAsString(VMControlVM *vm, const char *var);
char ** VMControl_VMWho(VMControlVM *vm);

/*
 * Callbacks
 */

typedef void (*VMControlVMCallback) (VMControlVM *vm, void* args);

typedef enum VMControlVMCallbackType {
   VMC_CALLBACK_POWER,
   VMC_CALLBACK_DIALOG,
   VMC_CALLBACK_ANSWER
} VMControlVMCallbackType;

Bool VMControl_VMRegisterCallback(VMControlVMCallbackType type, VMControlVMCallback c);

typedef struct VMControlPowerTransition {
   const char *username;
   const char *oldState;
   const char *newState;
} VMControlPowerTransition;

typedef struct VMControlQuestion {
   int sequence;
   const char * type; // info, warning, error, question
   const char * text;
   const char ** choices;
} VMControlQuestion;

typedef struct VMControlAnswer {
   VMControlQuestion *q;
   const char *username;
   int choice;
} VMControlAnswer;

#endif

